import sys
import click
import docker
import json
import requests

from higlass_manage.common import get_port
from higlass_manage.common import CONTAINER_PREFIX, REDIS_PREFIX

@click.command()
@click.option('--hg-name', default='default', 
        help='The name of the higlass container to import this file to')
def tilesets(hg_name):
    '''
    List the datasets in an instance
    '''
    port = get_port(hg_name)

    # use a really high limit to avoid paging
    url = 'http://localhost:{}/api/v1/tilesets/?limit=10000'.format(port)
    ret = requests.get(url)

    if ret.status_code != 200:
        sys.stderr.write('Error retrieving tilesets: {}'.format(ret))
        return

    j = json.loads(ret.content.decode('utf8'))
    for result in j['results']:
        sys.stdout.write("{}\n".format(" | ".join([result['uuid'], result['filetype'], result['datatype'], result['coordSystem'], result['name']])))

@click.command()
def instances():
    '''
    List running instances
    '''
    client = docker.from_env()

    for container in client.containers.list():
        name = container.name
        if name.find(CONTAINER_PREFIX) == 0:
            hm_name = name[len(CONTAINER_PREFIX)+1:]
            config = client.api.inspect_container(container.name)
            directories = " ".join( ['{}:{}'.format(m['Source'], m['Destination']) for m in  config['Mounts']])
            port = config['HostConfig']['PortBindings']['80/tcp'][0]['HostPort']
            sys.stdout.write("higlass\t{}\t{}\t{}\n".format(hm_name, directories, port))
        if name.find(REDIS_PREFIX) == 0:
            redis_name = name[len(REDIS_PREFIX)+1:]
            redis_config = client.api.inspect_container(container.name)
            redis_directories = " ".join( ['{}:{}'.format(m['Source'], m['Destination']) for m in redis_config['Mounts']])
            sys.stdout.write("redis\t{}\t{}\t.\n".format(redis_name, redis_directories))




import click
import subprocess as sp

from higlass_manage.common import hg_name_to_container_name

@click.command()
@click.argument('username')
@click.option('--hg-name', default='default', 
        help='The name of the higlass container to import this file to')
def superuser(username, hg_name):
    '''
    Delete a superuser in the container

    Parameters
    ----------
    hg_name: string
        The name of the container to create a superuser on
    '''
    container_name = hg_name_to_container_name(hg_name)
    proc_input = 'from django.contrib.auth.models import User; User.objects.get(username="{}").delete()'.format(username)
    sp.run(['docker', 'exec', '-i', container_name, 'python', 'higlass-server/manage.py', 'shell'], input=proc_input.encode('utf8'))


import click

from higlass_manage.ingest import ingest
from higlass_manage.list import tilesets, instances
from higlass_manage.start import start
from higlass_manage.stop import stop
from higlass_manage.shell import shell
from higlass_manage.view import view
from higlass_manage.logs import logs
from higlass_manage import __version__

import higlass_manage.create as hmce
import higlass_manage.delete as hmde

@click.command()
def version():
    print(__version__)

@click.group()
def list():
    pass

list.add_command(tilesets)
list.add_command(instances)

@click.group()
def create():
    pass

create.add_command(hmce.superuser)

@click.group()
def delete():
    pass

delete.add_command(hmde.superuser)

@click.group()
def cli():
    pass

cli.add_command(create)
cli.add_command(delete)
cli.add_command(list)
cli.add_command(ingest)
cli.add_command(start)
cli.add_command(stop)
cli.add_command(shell)
cli.add_command(view)
cli.add_command(logs)
cli.add_command(version)



import click
import docker
import subprocess as sp

from higlass_manage.common import hg_name_to_container_name

@click.command()
@click.argument('hg-name', nargs=-1)
def shell(hg_name):
    '''
    Start a shell in a higlass container
    '''
    if len(hg_name) == 0:
        hg_name = 'default'
    else:
        hg_name = hg_name[0]

    client = docker.from_env()
    container_name = hg_name_to_container_name(hg_name)
    container = client.containers.get(container_name)

    sp.run(['docker', 'exec', '-it', container_name, 'bash'])
    